//
//  ARKitViewController.swift
//  ios-app-bootstrap
//
//  Created by xdf on 05/11/2017.
//  Copyright © 2017 open source. All rights reserved.
//

import UIKit
import SceneKit
import ARKit

@available(iOS 11.0, *)
class ARKitViewController: UIViewController, ARSCNViewDelegate {
  var previousPoint: SCNVector3?
  var sceneView: ARSCNView!

  var lineColor = UIColor.yellow
  
  override func viewDidLoad() {
    super.viewDidLoad()
    initView()
  }
  
  func initView() {
    navigationItem.title = Utils.Path.basename(#file)
    view.backgroundColor = UIColor.white
    
    self.sceneView = ARSCNView(frame: self.view.frame)
    // Set the view's delegate
    sceneView.delegate = self
    
    // Show statistics such as fps and timing information
    sceneView.showsStatistics = true
    
    view.addSubview(sceneView)
    
    // Create a new scene
    let scene = SCNScene(named: "art.scnassets/world.scn")!
    
    // Set the scene to the view
    sceneView.scene = scene
  }
  
  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    
    // Create a session configuration
    let configuration = ARWorldTrackingConfiguration()
    
    // Run the view's session
    sceneView.session.run(configuration)
  }
  
  override func viewWillDisappear(_ animated: Bool) {
    super.viewWillDisappear(animated)
    
    // Pause the view's session
    sceneView.session.pause()
  }
  
  override func didReceiveMemoryWarning() {
    super.didReceiveMemoryWarning()
    // Release any cached data, images, etc that aren't in use.
  }
  
  // MARK: - ARSCNViewDelegate
  func renderer(_ renderer: SCNSceneRenderer, updateAtTime time: TimeInterval) {
  }

  func renderer(_ renderer: SCNSceneRenderer, willRenderScene scene: SCNScene, atTime time: TimeInterval) {
    
    guard let pointOfView = sceneView.pointOfView else { return }
    
    let mat = pointOfView.transform
    let dir = SCNVector3(-1 * mat.m31, -1 * mat.m32, -1 * mat.m33)
    let currentPosition = pointOfView.position + (dir * 0.1)
    if let previousPoint = previousPoint {
      let line = linePoint2Point(vector: previousPoint, toVector: currentPosition)
      let lineNode = SCNNode(geometry: line)
      lineNode.geometry?.firstMaterial?.diffuse.contents = lineColor
      sceneView.scene.rootNode.addChildNode(lineNode)
    }
    previousPoint = currentPosition
  }
  
  func session(_ session: ARSession, didFailWithError error: Error) {
    // Present an error message to the user
  }
  
  func sessionWasInterrupted(_ session: ARSession) {
    // Inform the user that the session has been interrupted, for example, by presenting an overlay
  }
  
  func sessionInterruptionEnded(_ session: ARSession) {
    // Reset tracking and/or remove existing anchors if consistent tracking is required
    
  }
  
  func linePoint2Point(vector vector1: SCNVector3, toVector vector2: SCNVector3) -> SCNGeometry {
    
    let indices: [Int32] = [0, 1]
    
    let source = SCNGeometrySource(vertices: [vector1, vector2])
    let element = SCNGeometryElement(indices: indices, primitiveType: .line)
    
    return SCNGeometry(sources: [source], elements: [element])
    
  }
}
